@@ -1,10 +1,94 @@
 /*
- * ospf_rxpkt.c,v 1.21.2.1 1993/11/11 02:43:50 jch Exp
+ * ------------------------------------------------------------------------
+ * 
+ * GateD, Release 3.5.5 
+ * 
+ * Copyright (c) 1996, 1997 The Regents of the University of Michigan
+ * All Rights Reserved
+ * 
+ * License to use, copy, modify, and distribute this software and its
+ * documentation can be obtained from Merit at the University of Michigan.
+ * 
+ * Merit GateDaemon Project
+ * 4251 Plymouth Road, Suite C
+ * Ann Arbor, MI 48105
+ * 
+ * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION WARRANTIES OF 
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE REGENTS OF THE
+ * UNIVERSITY OF MICHIGAN AND MERIT DO NOT WARRANT THAT THE
+ * FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET LICENSEE'S REQUIREMENTS OR
+ * THAT OPERATION WILL BE UNINTERRUPTED OR ERROR FREE. The Regents of the
+ * University of Michigan and Merit shall not be liable for
+ * any special, indirect, incidental or consequential damages with respect
+ * to any claim by Licensee or any third party arising from use of the
+ * software. GateDaemon was originated and developed through release 3.0
+ * by Cornell University and its collaborators.
+ * 
+ * Please forward bug fixes, enhancements and questions to the
+ * gated mailing list: gated-people@gated.merit.edu.
+ * 
+ * ---------------------
+ * 
+ * Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.
+ * All rights reserved.
+ * 
+ * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY
+ * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
+ * LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE.
+ * 
+ * Royalty-free licenses to redistribute GateD Release
+ * 3 in whole or in part may be obtained by writing to:
+ * 
+ * GateDaemon Project
+ * Information Technologies/Network Resources
+ * 200 CCC
+ * Cornell University
+ * Ithaca, NY  14853-2601  USA
+ * 
+ * GateD is based on Kirton's EGP, UC Berkeley's routing
+ * daemon	 (routed), and DCN's HELLO routing Protocol.
+ * Development of GateD has been supported in part by the
+ * National Science Foundation.
+ * 
+ * Please forward bug fixes, enhancements and questions to the
+ * gated mailing list: gated-people@gated.cornell.edu.
+ * 
+ * ------------------------------------------------------------------------
+ * 
+ * Portions of this software may fall under the following
+ * copyrights:
+ * 
+ * Copyright (c) 1988 Regents of the University of California.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms are
+ * permitted provided that the above copyright notice and
+ * this paragraph are duplicated in all such forms and that
+ * any documentation, advertising materials, and other
+ * materials related to such distribution and use
+ * acknowledge that the software was developed by the
+ * University of California, Berkeley.  The name of the
+ * University may not be used to endorse or promote
+ * products derived from this software without specific
+ * prior written permission.  THIS SOFTWARE IS PROVIDED
+ * ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * This copyright has ben automaticly added by the util/addcopyright.pl program.
+ * __END_OF_COPYRIGHT__
  */
 
-/* Gated Release 3.0 */
-/* Copyright (c) 1990,1991,1992,1993 by Cornell University. All rights reserved. */
-/* Refer to Particulars and other Copyright notices at the end of this file. */
+/*
+ * ospf_rxpkt.c,v 1.33.2.12 1995/01/23 12:40:56 jch Exp
+ */
+
+/* Gated Release 3.5 */
+/* Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.  All */
+/* rights reserved.  Refer to Particulars and other Copyright notices at */
+/* the end of this file.  */
+/*  */
 
 
 #define	INCLUDE_TIME
@@ -23,12 +107,11 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 		     router_id, sockaddr_un *,
 		     olen, size_t)
 {
-    struct LS_HDR *ap = &(ack->ack_piece);
+    struct LS_HDR *ap = &ack->ack_piece;
     struct LSDB *db = LSDBNULL;
     struct AREA *a, *area = intf->area;
-    struct LSDB_LIST *trans = LLNULL, *asetrans = LLNULL;
-    struct LSDB_LIST *lp;
-    int	ret_flag;
+    struct ospf_lsdb_list *trans = LLNULL, *asetrans = LLNULL;
+    struct ospf_lsdb_list *lp;
 
     if (nbr->state < NEXCHANGE) {
 	return OSPF_ERR_ACK_STATE;
@@ -37,11 +120,9 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
     /* list of db entries to be freed */
     for (olen -= OSPF_HDR_SIZE;
 	 olen > 0;
-	 olen -= ACK_PIECE_SIZE) {
+	 olen -= ACK_PIECE_SIZE, ap++) {
 	u_int diff;
 	
-	ret_flag = 0;
-
 	switch (ap->ls_type) {
 	case LS_RTR:
 	case LS_NET:
@@ -54,7 +135,6 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 	case LS_GM:
 	case LS_NSSA:
 	    /* Ignore */
-	    ap++;
 	    continue;
 
 	default:
@@ -65,21 +145,18 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 			     ap,
 			     "	BAD TYPE",
 			     ntohs(ap->ls_age));
-	    ap++;
 	    continue;
 	}
 	if (!(db = FindLSA(area, ap->ls_id, ap->adv_rtr, ap->ls_type))) {
-	    ap++;
 	    continue;
 	}
-	if (!DB_RETRANS(db)
+	if (!db->lsdb_retrans
 	    || !(lp = find_db_ptr(nbr, db))) {
-	    ap++;
 	    continue;
 	}
-	diff = DB_TIME(db) ? time_sec - DB_TIME(db) : 0;
+	diff = db->lsdb_time ? time_sec - db->lsdb_time : 0;
  	GNTOHS(ap->ls_age);
-	if (SAME_INSTANCE(ap, &(DB_RTR(db)->ls_hdr), diff)) {
+	if (SAME_INSTANCE(ap, &DB_RTR(db)->ls_hdr, diff)) {
 	    if (lp) {	/* on retrans list */
 		/* remove from lsdb */
 		(void) rem_nbr_ptr(db, nbr);
@@ -87,8 +164,8 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 		REM_DB_PTR(nbr, lp);
 
 		/* If MaxSeq and this is last ack, generate a new one */
-		if (DB_SEQ_MAX(db) == TRUE &&
-		    DB_RETRANS(db) == NLNULL &&
+		if (db->lsdb_seq_max == TRUE &&
+		    db->lsdb_retrans == NLNULL &&
 		    DB_FREEME(db) != TRUE)
 		    area->spfsched |= beyond_max_seq(area,
 						     intf,
@@ -102,10 +179,10 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 		 * retrans list - if empty, remove from lsdb
 		 */
 		if (DB_CAN_BE_FREED(db)) {
-		    db_free(db);
+		    db_free(db, LS_TYPE(db));
 		}
 	    }
-	} else if (MORE_RECENT(ap, &(DB_RTR(db)->ls_hdr), diff)) {
+	} else if (MORE_RECENT(ap, &DB_RTR(db)->ls_hdr, diff)) {
 	    OSPF_LOG_RX_LSA2(OSPF_ERR_ACK_BAD,
 			     intf,
 			     src,
@@ -120,7 +197,6 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 			     ap,
 			     db);
 	}
-	ap++;
     }
 
     if (asetrans != LLNULL) {	/* new self originated ls_ase */
@@ -129,11 +205,11 @@ ospf_rx_lsack  __PF7(ack, struct LS_ACK_HDR *,
 		self_orig_area_flood(a, asetrans, LS_ASE);
 	    }
 	} AREA_LIST_END(a) ;
-	ospf_freeq((struct Q **)&asetrans, OMEM_LL);
+	ospf_freeq((struct Q **)&asetrans, ospf_lsdblist_index);
     }
     if (trans != LLNULL) {
 	self_orig_area_flood(area, trans, LS_RTR);
-	ospf_freeq((struct Q **)&trans, OMEM_LL);
+	ospf_freeq((struct Q **)&trans, ospf_lsdblist_index);
     }
 
     return GOOD_RX;
@@ -160,7 +236,7 @@ db_parse __PF6(dbh, struct DB_HDR *,
     struct LS_REQ *lr;
     u_int diff;		/* to calculate db elapsed time */
 
-    dbp = &(dbh->dbp);
+    dbp = &dbh->dbp;
 
     for (; len; len -= DB_PIECE_SIZE) {
 	/* If we don't have a copy or if nbrs info is newer */
@@ -189,14 +265,16 @@ db_parse __PF6(dbh, struct DB_HDR *,
 	    (*(nbr_trans[SEQ_MISMATCH][nbr->state])) (intf, nbr);
 	    return 1;
 	}
-	if (db = FindLSA(a, dbp->ls_id, dbp->adv_rtr, dbp->ls_type))
-	    diff = time_sec - DB_TIME(db);
-	else
+	if ((db = FindLSA(a, dbp->ls_id, dbp->adv_rtr, dbp->ls_type))) {
+	    diff = time_sec - db->lsdb_time;
+	} else {
 	    diff = 0;
+	}
 	dbp->ls_age = ntohs(dbp->ls_age);
-	if (!db || MORE_RECENT(dbp, &(DB_RTR(db)->ls_hdr), diff)) {
+	if (!db
+	    || MORE_RECENT(dbp, &DB_RTR(db)->ls_hdr, diff)) {
 	    /* piece is newer or we don't have it - add to list */
-	    LS_REQ_ALLOC(lr);
+	    lr = (struct LS_REQ *) task_block_alloc(ospf_lsreq_index);
 	    lr->ls_id = dbp->ls_id;
 	    lr->adv_rtr = dbp->adv_rtr;
 	    lr->ls_age = htons(dbp->ls_age);
@@ -225,7 +303,7 @@ ospf_rx_db  __PF7(dbh, struct DB_HDR *,
     int reqcnt;
     struct DB_HDR *d;
     struct LSDB_SUM *nextds;
-    struct LSDB_LIST *txq = LLNULL;
+    struct ospf_lsdb_list *txq = LLNULL;
     struct AREA *area = intf->area;	/* area received from */
 
     if (BIT_MATCH(dbh->options, OPT_E_bit) == BIT_MATCH(area->area_flags, OSPF_AREAF_STUB))
@@ -302,82 +380,110 @@ ospf_rx_db  __PF7(dbh, struct DB_HDR *,
 	    goto we_are_through;
 	}
 	if (nbr->mode == MASTER) {
-	    if (nbr->seq == dbh->seq) {
-		if (nbr->dbsum == LSDB_SUM_NULL)
-		    d = (struct DB_HDR *) 0;
-		else
-		    d = &(nbr->dbsum->dbpkt->un.database);
 
-		/* if it's SLAVE's last and my last */
-		nbr->seq++;
-		/* parse packet and put on the nbrs link state req list */
-		if (olen) {
-		    if (db_parse(dbh, nbr, intf, src, dst, olen))
-			goto we_are_through;
-		}
-		if (!BIT_TEST(dbh->I_M_MS, bit_M) &&	/* slave's last */
-		    (d != (struct DB_HDR *) 0) &&	/* pkts left */
-		    !BIT_TEST(d->I_M_MS, bit_M)) {
-		    /* then, we're up to loading state */
-		    (*(nbr_trans[EXCH_DONE][nbr->state])) (intf, nbr);
-		    goto we_are_through;
-		}
-		/* free head of list f more bit was set */
-		if ((d != (struct DB_HDR *) 0) &&
-		    BIT_TEST(d->I_M_MS, bit_M)) {
-		    /* One less dbsum pkt */
-		    nbr->dbcnt -= nbr->dbsum->cnt;
-		    nextds = nbr->dbsum->next;
-		    DB_FREE_PKT(nbr->dbsum);
-		    FREE(nbr->dbsum, OMEM_DBSUM);
-		    nbr->dbsum = nextds;
-		}
-		send_dbsum(intf, nbr, NOT_RETRANS);
+	    /* Validate sequence number */
+	    switch (nbr->seq - dbh->seq) {
+	    case 0:
+		/* Expected sequence number */
+		break;
 
-	    } else if (nbr->seq - 1 != dbh->seq) {
+	    case 1:
+		/* Duplicate - ignore */
+		goto we_are_through;
+
+	    default:
+		/* Sequence mis-match */
 		(*(nbr_trans[SEQ_MISMATCH][nbr->state])) (intf, nbr);
+		goto we_are_through;
 	    }
-	} else {		/* we are in SLAVE mode  */
-	    if (nbr->seq + 1 == dbh->seq) {
-		nbr->seq = dbh->seq;
+	    
+	    if (nbr->dbsum == LSDB_SUM_NULL)
+		d = (struct DB_HDR *) 0;
+	    else
+		d = &nbr->dbsum->dbpkt->ospfh_un.database;
 
-		/* remove last sent */
-		if (nbr->dbsum != (struct LSDB_SUM *) 0) {
-		    /* One less dbsum pkt */
-		    nbr->dbcnt -= nbr->dbsum->cnt;
-		    nextds = nbr->dbsum->next;
-		    DB_FREE_PKT(nbr->dbsum);
-		    FREE(nbr->dbsum, OMEM_DBSUM);
-		    nbr->dbsum = nextds;
-		}
-		if (nbr->dbsum != (struct LSDB_SUM *) 0)
-		    d = &(nbr->dbsum->dbpkt->un.database);
-		else
-		    d = (struct DB_HDR *) NULL;
-
-		/* if it's MASTER's last and my last */
-		if (!BIT_TEST(dbh->I_M_MS, bit_M) &&	/* master's last */
-		    ((d == (struct DB_HDR *) NULL) ||	/* no pkts left */
-		     !BIT_TEST(d->I_M_MS, bit_M)))	/* my more bit is off */
-		    sched++;	/* schedule an exchange done event */
-
-		if (db_parse(dbh, nbr, intf, src, dst, olen))	/* run out of bufs */
+	    /* if it's SLAVE's last and my last */
+	    nbr->seq++;
+	    /* parse packet and put on the nbrs link state req list */
+	    if (olen) {
+		if (db_parse(dbh, nbr, intf, src, dst, olen))
 		    goto we_are_through;
-		send_dbsum(intf, nbr, NOT_RETRANS);
+	    }
+	    if (!BIT_TEST(dbh->I_M_MS, bit_M) &&	/* slave's last */
+		(d != (struct DB_HDR *) 0) &&	/* pkts left */
+		!BIT_TEST(d->I_M_MS, bit_M)) {
+		/* then, we're up to loading state */
+		(*(nbr_trans[EXCH_DONE][nbr->state])) (intf, nbr);
+		goto we_are_through;
+	    }
+	    /* free head of list f more bit was set */
+	    if ((d != (struct DB_HDR *) 0) &&
+		BIT_TEST(d->I_M_MS, bit_M)) {
+		/* One less dbsum pkt */
+		nbr->dbcnt -= nbr->dbsum->cnt;
+		nextds = nbr->dbsum->next;
+		dbsum_free(nbr->dbsum);
+		nbr->dbsum = nextds;
+	    }
+	    send_dbsum(intf, nbr, NOT_RETRANS);
+	} else {		/* we are in SLAVE mode  */
 
-		if (sched) {	/* up to loading state */
-		    (*(nbr_trans[EXCH_DONE][nbr->state])) (intf, nbr);
-		}
-	    } else if (nbr->seq == dbh->seq)
+	    /* Validate sequence number */
+	    switch (dbh->seq - nbr->seq) {
+	    case 1:
+		/* Expected sequence number */
+		break;
+
+	    case 0:
+		/* Duplicate - resent last packet */
 		send_dbsum(intf, nbr, NOT_RETRANS);
-	    else {
+		goto we_are_through;
+
+	    default:
+		/* Sequence mis-match */
 		(*(nbr_trans[SEQ_MISMATCH][nbr->state])) (intf, nbr);
+		goto we_are_through;
+	    }
+	    
+	    nbr->seq = dbh->seq;
+
+	    /* remove last sent */
+	    if (nbr->dbsum != (struct LSDB_SUM *) 0) {
+		/* One less dbsum pkt */
+		nbr->dbcnt -= nbr->dbsum->cnt;
+		nextds = nbr->dbsum->next;
+		dbsum_free(nbr->dbsum);
+		nbr->dbsum = nextds;
+	    }
+	    if (nbr->dbsum != (struct LSDB_SUM *) 0)
+		d = &nbr->dbsum->dbpkt->ospfh_un.database;
+	    else
+		d = (struct DB_HDR *) NULL;
+
+	    /* if it's MASTER's last and my last */
+	    if (!BIT_TEST(dbh->I_M_MS, bit_M) &&	/* master's last */
+		((d == (struct DB_HDR *) NULL) ||	/* no pkts left */
+		 !BIT_TEST(d->I_M_MS, bit_M)))	/* my more bit is off */
+		sched++;	/* schedule an exchange done event */
+
+	    if (db_parse(dbh, nbr, intf, src, dst, olen))	/* run out of bufs */
+		goto we_are_through;
+	    send_dbsum(intf, nbr, NOT_RETRANS);
+
+	    if (sched) {	/* up to loading state */
+		(*(nbr_trans[EXCH_DONE][nbr->state])) (intf, nbr);
 	    }
 	}
 	break;
 
     case NLOADING:
     case NFULL:
+	/* check for master mismatch */
+	if((BIT_MATCH(dbh->I_M_MS, bit_MS) == (nbr->mode == MASTER)) ||
+	BIT_TEST(dbh->I_M_MS, bit_I)) {
+		(*(nbr_trans[SEQ_MISMATCH][nbr->state]))(intf,nbr);
+		goto we_are_through;
+	}
 	if (nbr->mode == SLAVE_HOLD && nbr->seq == dbh->seq)
 	    /* haven't timed out yet */
 	    send_dbsum(intf, nbr, NOT_RETRANS);
@@ -385,6 +491,17 @@ ospf_rx_db  __PF7(dbh, struct DB_HDR *,
 		 (nbr->mode == SLAVE && nbr->seq == dbh->seq))
 	    goto we_are_through;
 	else {
+	    if (nbr->state == NFULL) {
+		trace_log_tf(ospf.trace_options,
+			     0,
+			     LOG_WARNING,
+			     ("Lost %s Neighbor %A with address %A due to sequence mismatch (%d versus %d).",
+			      ((nbr->mode == MASTER) ? "Master" : "Slave"),
+			      nbr->nbr_id, 
+			      nbr->nbr_addr,
+			      ((nbr->mode == MASTER) ? (nbr->seq-1) : nbr->seq),
+			      dbh->seq));
+	    }
 	    (*(nbr_trans[SEQ_MISMATCH][nbr->state])) (intf, nbr);
 	}
 
@@ -407,7 +524,7 @@ ospf_rx_db  __PF7(dbh, struct DB_HDR *,
     }
     if (txq != LLNULL) {	/* may be locked out */
 	self_orig_area_flood(area, txq, LS_RTR);
-	ospf_freeq((struct Q **)&txq, OMEM_LL);
+	ospf_freeq((struct Q **)&txq, ospf_lsdblist_index);
     }
 
     /* Can we send req pkt? */
@@ -426,20 +543,21 @@ ospf_rx_db  __PF7(dbh, struct DB_HDR *,
 
 
 static int
-ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
+ospf_rx_hello  __PF8(hello, struct HELLO_HDR *,
+		     newnbr, struct NBR *,
 		     intf, struct INTF *,
 		     src, sockaddr_un *,
 		     dst, sockaddr_un *,
 		     router_id, sockaddr_un *,
-		     olen, size_t)
+		     olen, size_t,
+		     sequence, u_int32)
 {
     int 	nbr_is_dr = FALSE, nbr_is_bdr = FALSE;
     int		rtr_heard_from = FALSE;
     int		backup_seen = FALSE;
-    struct 	NBR *nbr;
-    struct 	NBR *newnbr = NBRNULL;
-    struct 	RHF *rhf, *last_rhf;
-    struct 	LSDB_LIST *txq = LLNULL;
+    byte *	limit = (byte *) hello + olen - OSPF_HDR_SIZE;
+    struct 	RHF *rhf;
+    struct 	ospf_lsdb_list *txq = LLNULL;
     struct 	AREA *area = intf->area;	/* area received from */
 
     if (intf->state == IDOWN) {
@@ -454,120 +572,119 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
      * a few validity checks 
      */
     if (intf->type != VIRTUAL_LINK
-	&& hello->netmask != INTF_MASK(intf))
+	&& intf->type != POINT_TO_POINT
+	&& hello->netmask != INTF_MASK(intf)) {
 	return OSPF_ERR_HELLO_MASK;
-    if (ntohs(hello->HelloInt) != intf->hello_timer)
-	return OSPF_ERR_HELLO_TIMER;
-    if (ntohl(hello->DeadInt) != intf->dead_timer)
-	return OSPF_ERR_HELLO_DEAD;
-    if (BIT_MATCH(hello->options, OPT_E_bit) == BIT_MATCH(area->area_flags, OSPF_AREAF_STUB))
-	return OSPF_ERR_HELLO_E;
-
-    /*
-     * for multi-access nets find the nbr that origniated this hello
-     */
-    if (intf->type <= NONBROADCAST) {
-	for (nbr = intf->nbr.next; nbr != NBRNULL; nbr = nbr->next) {
-	    if (!NBR_ADDR(nbr))
-		continue;
-	    if (sockaddrcmp_in(src, nbr->nbr_addr)) {
-		newnbr = nbr;
-		break;
-	    }
-	}
     }
- 
-    if (!newnbr) {
-	/* New neighbor */
+    if (ntohs(hello->HelloInt) != intf->hello_timer) {
+	return OSPF_ERR_HELLO_TIMER;
+    }
+    if (ntohl(hello->DeadInt) != intf->dead_timer) {
+	return OSPF_ERR_HELLO_DEAD;
+    }
+    if (BIT_MATCH(hello->options, OPT_E_bit) == BIT_MATCH(area->area_flags, OSPF_AREAF_STUB)) {
+	return OSPF_ERR_HELLO_E;
+    }
+    if (sockaddrcmp_in(router_id, ospf.router_id)) {
+	return OSPF_ERR_HELLO_ID;
+    }
 
-	if (sockaddrcmp_in(router_id, ospf.router_id)) {
-	    return OSPF_ERR_HELLO_ID;
-	}
-	
-	switch (intf->type) {
-	case POINT_TO_POINT:
-	    newnbr = &(intf->nbr);
-	    if (newnbr->nbr_addr) {
-		sockfree(newnbr->nbr_addr);
+    switch (intf->type) {
+    case POINT_TO_POINT:
+	if (!newnbr) {
+	    newnbr = &intf->nbr;
+	    assert(sockaddrcmp_in(src, newnbr->nbr_addr));
+#ifdef	notdef
+	    if (!sockaddrcmp_in(src, newnbr->nbr_addr)) {
+		if (newnbr->nbr_addr) {
+		    sockfree(newnbr->nbr_addr);
+		}
+		newnbr->nbr_addr = sockdup(src);
 	    }
-	    newnbr->nbr_addr = sockdup(src);
-	    break;
+#endif	/* notdef */
+	    newnbr->nbr_sequence = sequence;
+	    newnbr->intf = intf;
+	    newnbr->pri = hello->rtr_priority;
+	    if (newnbr->nbr_id) {
+		sockfree(newnbr->nbr_id);
+	    }
+	    newnbr->nbr_id = sockdup(router_id);
+	}
+	break;
 
-	case VIRTUAL_LINK:
-	    newnbr = &(intf->nbr);
-	    break;
+    case VIRTUAL_LINK:
+	assert(newnbr);		/* Neighbor is found when we find virtual link */
+	break;
+	
+    case NONBROADCAST:
+    case BROADCAST:
+	if (newnbr) {
+	    /* Update neighbor ID */
+
+	    if (!newnbr->nbr_id
+		|| !sockaddrcmp_in(newnbr->nbr_id, router_id)) {
+		if (newnbr->nbr_id) {
+		    sockfree(newnbr->nbr_id);
+		}
+		newnbr->nbr_id = sockdup(router_id);
+	    }
+	} else {
+	    /* Allocate a new neighbor */
+
+	    if (intf->type == NONBROADCAST && intf->pri) {
+		/* NBMA DR elig nbrs should be configured */
 
-	case NONBROADCAST:
-	    /* NBMA DR elig nbrs should be configured */
-	    if (intf->pri) {
 		OSPF_LOG_RX(OSPF_ERR_HELLO_NBMA,
 			    intf,
 			    src,
 			    dst);
 	    }
-	    /* Fall through */
 
-	case BROADCAST:
-	    NBR_ALLOC(newnbr);
-	    if (!newnbr) {
-		return GOOD_RX;
-	    }
-	    newnbr->nbr_addr = sockdup(src);
-	    nbr_enq(intf,newnbr);
-	    break;
-	}
-	if (intf->type != VIRTUAL_LINK) {
-	    if (newnbr->nbr_id) {
-		sockfree(newnbr->nbr_id);
-	    }
+	    newnbr = (struct NBR *) task_block_alloc(ospf_nbr_index);
 	    newnbr->nbr_id = sockdup(router_id);
-#ifdef	notdef
-	    if (newnbr->ifap) {
-		ifa_free(newnbr->ifap);
-	    }
-	    newnbr->ifap = ifa_alloc(intf->ifap);
-#endif	/* notdef */
+	    newnbr->nbr_addr = sockdup(src);
+	    newnbr->nbr_sequence = sequence;
 	    newnbr->intf = intf;
 	    newnbr->pri = hello->rtr_priority;
-	} else if (!sockaddrcmp_in(newnbr->nbr_id, router_id)) {
-	    return OSPF_ERR_HELLO_VIRT;
-	}
-    } else {
-	/* Old neighbor */
-
-	if (!newnbr->nbr_id ||
-	    !sockaddrcmp_in(newnbr->nbr_id, router_id)) {
-	    /* No neighbor ID, or it changed */
-	    if (newnbr->nbr_id) {
-		sockfree(newnbr->nbr_id);
-	    }
-	    newnbr->nbr_id = sockdup(router_id);
+	    ospf_nbr_add(intf,newnbr);
 	}
+	break;
     }
-    reset_inact_tmr(newnbr);		/* if not set don't set */
 
+    reset_inact_tmr(newnbr);		/* if not set don't set */
 
     /* 
      * have already reset rtr dead timer or will start in NHello 
      */
-    if (intf->type == VIRTUAL_LINK || intf->type == NONBROADCAST) {
-	if ((newnbr->state == NATTEMPT) || (newnbr->state == NDOWN))
+    switch (intf->type) {
+    case VIRTUAL_LINK:
+    case NONBROADCAST:
+	switch (newnbr->state) {
+	case NATTEMPT:
+	case NDOWN:
 	    (*(nbr_trans[HELLO_RX][newnbr->state])) (intf, newnbr);
-    } else if (newnbr->state == NDOWN)
-	(*(nbr_trans[HELLO_RX][newnbr->state])) (intf, newnbr);
+	    break;
+
+	default:
+	    break;
+	}
+	break;
+
+    default:
+	if (newnbr->state == NDOWN) {
+	    (*(nbr_trans[HELLO_RX][newnbr->state])) (intf, newnbr);
+	}
+	break;
+    }
 
     /* 
-     * see if THIS rtr appears in the router-heard-from list 
+     * see if my router_id appears in the router-heard-from list 
      */
-    last_rhf = (struct RHF *) ((u_int32) & (hello->rhf) +
-			       (olen - (OSPF_HDR_SIZE + HELLO_HDR_SIZE)));
-
-    for (rhf = (struct RHF *) & (hello->rhf);
-	 rhf <  last_rhf;
-	 rhf++) {
+    for (rhf = &hello->rhf; (byte *) rhf < limit; rhf++) {
 	if (rhf->rtr == MY_ID) {
-	    if (newnbr->state < N2WAY)
+	    if (newnbr->state < N2WAY) {
 		(*(nbr_trans[TWOWAY][newnbr->state])) (intf, newnbr);
+	    }
 	    rtr_heard_from++; 		/* flag that we found it */
 	    break;
 	}
@@ -575,6 +692,14 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
 
     /* drop back to one way until we can see ourself in this nbr's hello */
     if (!rtr_heard_from) {
+	if (newnbr->state == NFULL) {
+	    trace_log_tf(ospf.trace_options,
+			 0,
+			 LOG_WARNING,
+			 ("Lost Neighbor %A with address %A due to HELLO received without my ID.",
+			  newnbr->nbr_id, 
+			  newnbr->nbr_addr));
+	}
 	(*(nbr_trans[ONEWAY][newnbr->state])) (intf, newnbr);
     } else if (intf->type <= NONBROADCAST) {
 
@@ -587,9 +712,9 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
 	/* 
 	 * if this nbr is bdr or it's dr and there is no bdr 
 	 */
-	if ((intf->state == IWAITING) &&
-	    (((nbr_is_dr && !(hello->bdr))) || (nbr_is_bdr))) 
-	{
+	if (intf->state == IWAITING
+	    && ((nbr_is_dr && !hello->bdr)
+		|| nbr_is_bdr)) {
 		newnbr->dr = hello->dr;
                 newnbr->bdr = hello->bdr;
 		backup_seen = TRUE;
@@ -598,12 +723,15 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
 	    if (intf->state >= IDr)
 		BIT_SET(intf->flags, OSPF_INTFF_NBR_CHANGE);	/* schedule nbr change */
     	} else {	/* dr/bdr changeola? */
-    	    if ((hello->dr != newnbr->dr) &&
-      	    	((newnbr->dr == NBR_ADDR(newnbr)) || (nbr_is_dr)))
+    	    if (hello->dr != newnbr->dr
+		&& (newnbr->dr == NBR_ADDR(newnbr)
+		    || nbr_is_dr)) {
 		BIT_SET(intf->flags, OSPF_INTFF_NBR_CHANGE);
-    	    else if ((hello->bdr != newnbr->bdr) &&
-      	    	((newnbr->bdr == NBR_ADDR(newnbr)) || (nbr_is_bdr)))
+    	    } else if (hello->bdr != newnbr->bdr
+		       && (newnbr->bdr == NBR_ADDR(newnbr)
+			   || nbr_is_bdr)) {
 		BIT_SET(intf->flags, OSPF_INTFF_NBR_CHANGE);
+	    }
 	}
 
 	newnbr->dr = hello->dr;
@@ -612,12 +740,14 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
 
 
     /* If not elig send hello to elig nbr if nbr isn't dr or bdr */
-    if (intf->type == NONBROADCAST &&
-	(!intf->pri) &&
-	newnbr->pri &&
-	intf->dr != newnbr &&
-	intf->bdr != newnbr)
+    if (intf->type == NONBROADCAST
+	&& !intf->pri
+	&& newnbr->pri
+	&& intf->dr != newnbr
+	&& intf->bdr != newnbr) {
+
 	send_hello(intf, newnbr, FALSE);
+    }
 
     if (backup_seen) {
 	(*(if_trans[BACKUP_SEEN][intf->state])) (intf);
@@ -636,7 +766,7 @@ ospf_rx_hello  __PF6(hello, struct HELLO_HDR *,
     }
     if (txq != LLNULL) {		/* may be locked out */
 	self_orig_area_flood(area, txq, LS_RTR);
-	ospf_freeq((struct Q **) &txq, OMEM_LL);
+	ospf_freeq((struct Q **) &txq, ospf_lsdblist_index);
     }
 
     return GOOD_RX;
@@ -666,15 +796,15 @@ ospf_rx_lsreq  __PF7(ls_req, struct LS_REQ_HDR *,
     struct LS_REQ_PIECE *req = &ls_req->req;
     struct AREA *a = intf->area;
     struct LSDB *db;
-    struct LSDB_LIST *txq = LLNULL;
+    struct ospf_lsdb_list *txq = LLNULL;
     u_int32 *adv_cnt;
     union ADV *adv;
     size_t len = OSPF_HDR_SIZE + 4;/* max len of pkt is INTF_MTU */
-    int intf_mtu = INTF_MTU(intf);
+    size_t intf_mtu = INTF_MTU(intf);
     int ret = GOOD_RX;
 
     if (nbr->state < NEXCHANGE) {
-	ret = OSPF_ERR_ACK_STATE;
+	return OSPF_ERR_REQ_STATE;
     }
 
     olen -= OSPF_HDR_SIZE;
@@ -689,13 +819,24 @@ ospf_rx_lsreq  __PF7(ls_req, struct LS_REQ_HDR *,
     for (; olen; olen -= LS_REQ_PIECE_SIZE) {
 	if ((req->ls_type < LS_RTR || req->ls_type > LS_ASE) ||
 	    !(db = FindLSA(a, req->ls_id, req->adv_rtr, req->ls_type))) {
+	    if (nbr->state == NFULL) {
+		trace_log_tf(ospf.trace_options,
+			     0,
+			     LOG_WARNING,
+			     ("Lost Neighbor %A with address %A due to bad LS Request (%A %A %d).",
+			      nbr->nbr_id, 
+			      nbr->nbr_addr,
+			      sockbuild_in(0, req->ls_id),
+			      sockbuild_in(0, req->adv_rtr),
+			      req->ls_type));
+	    }
 	    (*(nbr_trans[BAD_LS_REQ][nbr->state])) (intf, nbr);
 	    ret = OSPF_ERR_REQ_BOGUS;
 	    goto we_are_through;
 	}
 	if ((len + ntohs(LS_LEN(db))) > intf_mtu) {
 	    GHTONL(*adv_cnt);
-	    ospf_txpkt(pkt, intf, O_LSU, len, NBR_ADDR(nbr), NOT_RETRANS);
+	    ospf_txpkt(pkt, intf, OSPF_PKT_LSU, len, NBR_ADDR(nbr), NOT_RETRANS);
 	    len = OSPF_HDR_SIZE + 4;	/* 4 for adv_cnt */
 	    NEW_LSU(pkt, adv_cnt, adv);
 	}
@@ -709,7 +850,7 @@ ospf_rx_lsreq  __PF7(ls_req, struct LS_REQ_HDR *,
 
     if (*adv_cnt) {		/* any left to send? */
 	GHTONL(*adv_cnt);
-	ospf_txpkt(pkt, intf, O_LSU, len, NBR_ADDR(nbr), FALSE);
+	ospf_txpkt(pkt, intf, OSPF_PKT_LSU, len, NBR_ADDR(nbr), FALSE);
     }
 
   we_are_through:
@@ -729,7 +870,7 @@ ospf_rx_lsreq  __PF7(ls_req, struct LS_REQ_HDR *,
     }
     if (txq != LLNULL) {		/* may be locked out */
 	self_orig_area_flood(a, txq, LS_RTR);
-	ospf_freeq((struct Q **) &txq, OMEM_LL);
+	ospf_freeq((struct Q **) &txq, ospf_lsdblist_index);
     }
 
     return ret;
@@ -746,12 +887,11 @@ ospf_rxpkt __PF4(ip, struct ip *,
 {
     size_t len;
     int ret = 0;
-    u_int32 *auth_key;
-    u_int16 auth_type;
-    struct AREA *area = AREANULL;
-    flag_t spf_flag = OSPF_AREAF_SPF_RUN|OSPF_AREAF_SPF_FULL;
-    sockaddr_un *router_id = sockbuild_in(0, o_hdr->rtr_id);
+    ospf_auth *oaps[3], **oapp = oaps, *oap;
+    u_int32 sequence;
+    sockaddr_un *router_id = sockbuild_in(0, o_hdr->ospfh_rtr_id);
     struct INTF *intf;
+    struct NBR *nbr = (struct NBR *) 0;
 #define	bad_packet(x)	{ ret = x; goto Return; }
 
     /* Log current time */
@@ -760,7 +900,7 @@ ospf_rxpkt __PF4(ip, struct ip *,
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
 
-    if (o_hdr->type == O_MON) {
+    if (o_hdr->ospfh_type == OSPF_PKT_MON) {
 	/* Monitor packet */
 
 	intf = (struct INTF *) 0;
@@ -847,11 +987,11 @@ ospf_rxpkt __PF4(ip, struct ip *,
 
     }
 
-    if (IP_PROTOCOL(ip) != IPPROTO_OSPF) {
+    if (IP_PROTOCOL(ip) != ospf.task->task_proto) {
 	bad_packet(OSPF_ERR_IP_PROTO);
     }
 
-    len = ntohs(o_hdr->length);
+    len = ntohs(o_hdr->ospfh_length);
     if (len < OSPF_HDR_SIZE) {
 	bad_packet(OSPF_ERR_OSPF_SHORT);
     }
@@ -860,23 +1000,31 @@ ospf_rxpkt __PF4(ip, struct ip *,
     }
 
     /* OSPF hdr */
-    if (o_hdr->version != OSPF_VERSION) {
+    if (o_hdr->ospfh_version != OSPF_VERSION) {
 	bad_packet(OSPF_ERR_OSPF_VERSION);
     }
 
     /* Checksum the packet, exclusive of the authentication field */
-    {
+    if (ntohs(o_hdr->ospfh_auth_type) == OSPF_AUTH_MD5) {
+	/* With MD5 the checksum field must be zero */
+	
+	if (o_hdr->ospfh_checksum) {
+	    bad_packet(OSPF_ERR_OSPF_CHKSUM);
+	}
+    } else {
 	struct iovec v[2], *vp = v;
 
+	/* Verify the INET checksum */
+	
 	/* First the packet header not including the authentication */
 	vp->iov_base = (caddr_t) o_hdr;
-	vp->iov_len = OSPF_HDR_SIZE - OSPF_AUTH_SIZE;
+	vp->iov_len = OSPF_HDR_SIZE - OSPF_AUTH_SIMPLE_SIZE;
 	vp++;
 
 	if (len > OSPF_HDR_SIZE) {
 	    /* Then the rest of the packet */
 
-	    vp->iov_base = (caddr_t) o_hdr->Auth + OSPF_AUTH_SIZE;
+	    vp->iov_base = (caddr_t) o_hdr->ospfh_auth_key + OSPF_AUTH_SIMPLE_SIZE;
 	    vp->iov_len = len - OSPF_HDR_SIZE;
 	    vp++;
 	} else {
@@ -886,23 +1034,55 @@ ospf_rxpkt __PF4(ip, struct ip *,
 	    vp->iov_base = 0;
 	}
 
-	if (inet_cksumv(v, vp - v, len - OSPF_AUTH_SIZE)) {
+	if (inet_cksumv(v, vp - v, len - OSPF_AUTH_SIMPLE_SIZE)) {
 	    bad_packet(OSPF_ERR_OSPF_CHKSUM);
 	}
     }
 
-    if (o_hdr->type == O_MON) {
-	auth_type = ospf.mon_authtype;
-	auth_key = ospf.mon_authkey;
+    if (o_hdr->ospfh_type == OSPF_PKT_MON) {
+	*oapp++ = &ospf.mon_auth;
+	nbr = (struct NBR *) 0;
     } else {
 	/* the area id check stuff - VIRTUAL or Standard link? */
 	if (intf
-	    && o_hdr->area_id == intf->area->area_id) {
+	    && o_hdr->ospfh_area_id == intf->area->area_id) {
 	    /* NOT virtual */
 
 	    if ((sock2ip(src) & INTF_MASK(intf)) != (INTF_ADDR(intf) & INTF_MASK(intf)))
 		bad_packet(OSPF_ERR_OSPF_AREAID);
-	    area = intf->area;
+
+	    /* Find the neighbor */
+	    switch (intf->type) {
+	    case BROADCAST:
+	    case NONBROADCAST:
+		/* Look up neighbor in tree */
+	    
+		OSPF_NBR_LOOKUP(nbr, intf, src);
+		break;
+
+	    case POINT_TO_POINT:
+		nbr = FirstNbr(intf);
+		if (!nbr->nbr_id
+		    || !sockaddrcmp_in(nbr->nbr_id, router_id)) {
+		    /* Unknown router ID, or unknown address */
+
+		    nbr = (struct NBR *) 0;
+		}
+		break;
+
+	    default:
+		assert(FALSE);
+	    }
+
+	    if (!nbr && o_hdr->ospfh_type != OSPF_PKT_HELLO) {
+		/* Unknown neighbor */
+	    
+		if (OSPF_LOG_TIME(intf)) {
+		    bad_packet(OSPF_ERR_OSPF_NBR);
+		} else {
+		    return;
+		}
+	    }
 	} else {
 	    struct INTF *vintf;
 
@@ -917,133 +1097,179 @@ ospf_rxpkt __PF4(ip, struct ip *,
 		 * rtr 3) trans area is same as receiving IF's area
 		 */
 		if (sockaddrcmp_in(vintf->nbr.nbr_id, router_id) &&
-		    o_hdr->area_id == OSPF_BACKBONE &&	/* from backbone */
-		    vintf->transarea == intf->area) {
-		    area = &ospf.backbone;
-		    break;
+		    o_hdr->ospfh_area_id == OSPF_BACKBONE &&	/* from backbone */
+		    vintf->trans_area == intf->area) {
+		    intf = vintf;
+		    nbr = &intf->nbr;
+		    goto found_intf;
 		}
 	    } VINTF_LIST_END(vintf) ;
 
-	    if (!(intf = vintf)) {
-		bad_packet(OSPF_ERR_OSPF_VL);
+	    /* Not found */
+	    bad_packet(OSPF_ERR_OSPF_VL);
+
+	found_intf: ;
+	}
+
+	*oapp++ = &intf->auth;
+	if (BIT_TEST(intf->flags, OSPF_INTFF_SECAUTH)) {
+	    *oapp++ = &intf->auth2;
+	}
+    }
+
+    *oapp = (ospf_auth *) 0;
+
+    {
+	size_t md5_done = 0;
+	u_int32 md5_temp[OSPF_AUTH_SIZE];
+	
+	for (oapp = oaps; (oap = *oapp); oapp++ ) {
+	
+	    /* Check the authentication */
+	    if (ntohs(o_hdr->ospfh_auth_type) != oap->auth_type) {
+		continue;
+	    }
+
+	    switch (oap->auth_type) {
+	    case OSPF_AUTH_NONE:
+		sequence = (u_int32) 0;
+		goto auth_ok;
+
+	    case OSPF_AUTH_SIMPLE:
+		if (o_hdr->ospfh_auth_key[0] == oap->auth_key[0]
+		    && o_hdr->ospfh_auth_key[1] == oap->auth_key[1]) {
+		    sequence = (u_int32) 0;
+		    goto auth_ok;
+		}
+		break;
+
+	    case OSPF_AUTH_MD5:
+		if (htons(o_hdr->ospfh_md5_offset) + OSPF_AUTH_MD5_SIZE <= (size_t) IP_LENGTH(ip)
+		    && !o_hdr->ospfh_md5_instance) {
+		    u_int32 *dp = (u_int32 *) ((void_t) ((byte *) o_hdr + ntohs(o_hdr->ospfh_md5_offset)));
+		    u_int32 digest[OSPF_AUTH_SIZE];
+
+		    /* Save the digest */
+		    digest[0] = dp[0]; digest[1] = dp[1]; digest[2] = dp[2]; digest[3] = dp[3];
+
+		    /* Checksum the packet up to the digest */
+		    if (!md5_done) {
+			md5_done = md5_cksum_partial(o_hdr,
+						     dp,
+						     TRUE,
+						     md5_temp);
+		    }
+
+		    /* Put the secret in the packet */
+		    dp[0] = oap->auth_key[0]; dp[1] = oap->auth_key[1];
+		    dp[2] = oap->auth_key[2]; dp[3] = oap->auth_key[3];
+
+		    md5_cksum((byte *) o_hdr + md5_done, 
+			      len + OSPF_AUTH_MD5_SIZE - md5_done,
+			      len + OSPF_AUTH_MD5_SIZE,
+			      dp,
+			      md5_temp);
+
+		    if (digest[0] == dp[0]
+			&& digest[1] == dp[1]
+			&& digest[2] == dp[2]
+			&& digest[3] == dp[3]) {
+			/* Verify that the sequence is not less than and save the */
+			/* new sequence number */
+
+			/* XXX - Handle wrap? */
+			sequence = ntohl(o_hdr->ospfh_md5_sequence);
+			if (nbr) {
+			    if (sequence < nbr->nbr_sequence) {
+				/* Sequence not valid */
+				
+				continue;
+			    }
+			    nbr->nbr_sequence = sequence;
+			}
+			goto auth_ok;
+		    }
+		}
+		break;
+	
+	    default:
+		bad_packet(OSPF_ERR_OSPF_AUTH_TYPE);
 	    }
 	}
 
-	auth_type = area->authtype;
-	auth_key = intf->authkey;
+	/* Search of keys has failed */
+	bad_packet(OSPF_ERR_OSPF_AUTH_KEY);
+
+    auth_ok: ;
     }
-
-    /* Check the authentication */
-    if (ntohs(o_hdr->AuType) != auth_type)
-	bad_packet(OSPF_ERR_OSPF_AUTH_TYPE);
-
-    switch (auth_type) {
-    case OSPF_AUTH_NONE:
-	break;
-
-    case OSPF_AUTH_SIMPLE:
-	if (o_hdr->Auth[0] != auth_key[0] ||
-	    o_hdr->Auth[1] != auth_key[1]) {
-	    bad_packet(OSPF_ERR_OSPF_AUTH_KEY);
-	}
-	break;
-
-    default:
-	bad_packet(OSPF_ERR_OSPF_AUTH_TYPE);
-    }
-
-    /* Log the packet */
-    if (o_hdr->type != O_MON &&
-	BIT_TEST(ospf.trace_flags, TR_OSPF_PKT)) {
-	ospf_trace(o_hdr, len, (u_int) o_hdr->type, 1, intf, src, dst);
-    }
-
     
+    /* Log the packet */
+    if (TRACE_PACKET_RECV(ospf.trace_options,
+			  o_hdr->ospfh_type,
+			  OSPF_PKT_MAX,
+			  ospf_trace_masks)) {
+	ospf_trace(o_hdr,
+		   len,
+		   (u_int) o_hdr->ospfh_type,
+		   TRUE,
+		   intf,
+		   src,
+		   dst,
+		   TRACE_DETAIL_RECV(ospf.trace_options,
+				     o_hdr->ospfh_type,
+				     OSPF_PKT_MAX,
+				     ospf_trace_masks));
+    }
+
     /* Handle the packet */
-    switch (o_hdr->type) {
-	struct NBR *nbr;
+    switch (o_hdr->ospfh_type) {
+    case OSPF_PKT_MON:
+	ret = ospf_rx_mon(&o_hdr->ospfh_un.mon, intf, src, router_id, len);
+	break;
+
+    case OSPF_PKT_HELLO:
+	ret = ospf_rx_hello(&o_hdr->ospfh_un.hello,
+			    nbr,
+			    intf,
+			    src,
+			    dst,
+			    router_id,
+			    len,
+			    sequence);
+	break;
+	    
+    case OSPF_PKT_DD:
+	ret = ospf_rx_db(&o_hdr->ospfh_un.database, nbr, intf, src, dst, router_id, len);
+	break;
 	
-      case O_MON:
-  	ret = ospf_rx_mon(&o_hdr->un.mon, intf, src, router_id, len);
-	return;
+    case OSPF_PKT_LSR:
+	ret = ospf_rx_lsreq(&o_hdr->ospfh_un.ls_req, nbr, intf, src, dst, router_id, len);
+	break;
+	
+    case OSPF_PKT_LSU:
+	ret = ospf_rx_lsupdate(&o_hdr->ospfh_un.ls_update, nbr, intf, src, dst, router_id, len);
+	break;
+
+    case OSPF_PKT_ACK:
+	ret = ospf_rx_lsack(&o_hdr->ospfh_un.ls_ack, nbr, intf, src, dst, router_id, len);
+	break;
 
-      case O_HELLO:
-	ret = ospf_rx_hello(&o_hdr->un.hello, intf, src, dst, router_id, len);
-  	break;
-  	    
     default:
 	bad_packet(OSPF_ERR_OSPF_TYPE);
-  	
-    case O_DB_DESCRIPT:
-    case O_LSR:
-    case O_ACK:
-    case O_LSU:
-	/* Locate neighbor */
-	if (intf->type > NONBROADCAST) {
-	    nbr = FirstNbr(intf);
-	    if (!nbr->nbr_id
-		|| !sockaddrcmp_in(nbr->nbr_id, router_id)) {
-		nbr = NBRNULL;
-	    }
-	} else {
-	    NBRS_LIST(nbr, intf) {
-		if (sockaddrcmp_in(src, nbr->nbr_addr)) {
-		    break;
-		}
-	    } NBRS_LIST_END(nbr, intf) ;
-	}
-  
-	if (!nbr) {
-	    if (OSPF_LOG_TIME(intf)) {
-		bad_packet(OSPF_ERR_OSPF_NBR);
-	    } else {
-		return;
-	    }
-	}
-  
-	switch (o_hdr->type) {
-	case O_DB_DESCRIPT:
-	    ret = ospf_rx_db(&o_hdr->un.database, nbr, intf, src, dst, router_id, len);
-	    break;
-	
-	case O_LSR:
-	    ret = ospf_rx_lsreq(&o_hdr->un.ls_req, nbr, intf, src, dst, router_id, len);
-	    break;
-	
-	case O_LSU:
-	    BIT_RESET(spf_flag, OSPF_AREAF_SPF_FULL);
-	    ret = ospf_rx_lsupdate(&o_hdr->un.ls_update, nbr, intf, src, dst, router_id, len);
-	    break;
-  
-	case O_ACK:
-	    ret = ospf_rx_lsack(&o_hdr->un.ls_ack, nbr, intf, src, dst, router_id, len);
-	    break;
-
-	default:
-	    assert(FALSE);
-	}
-	break;
     }
 	
-    
     if (ret == GOOD_RX) {
-	ret = o_hdr->type;
-    }
-
-    /* Schedule an SPF run if necessary */
-    if (area && area->spfsched) {
-
-	BIT_SET(area->area_flags, spf_flag);
+	ret = o_hdr->ospfh_type;
     }
 
  Return:
-    if (ret != OSPF_ERR_IGNORE) {
-	OSPF_LOG_RX(ret, intf, src, dst);
-    }
+    OSPF_LOG_RX(ret, intf, src, dst);
 
     if (ospf_log_last_lsa) {
 	ospf_log_last_lsa = 0;
-	trace(TR_OSPF, 0, (NULL));
+	trace_only_tf(ospf.trace_options,
+		      0,
+		      (NULL));
     }
 }
 
@@ -1051,9 +1277,9 @@ ospf_rxpkt __PF4(ip, struct ip *,
 /*
  * ------------------------------------------------------------------------
  * 
- * 	GateD, Release 3
+ * 	GateD, Release 3.5
  * 
- * 	Copyright (c) 1990,1991,1992,1993 by Cornell University
+ * 	Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.
  * 	    All rights reserved.
  * 
  * 	THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY
@@ -1066,7 +1292,7 @@ ospf_rxpkt __PF4(ip, struct ip *,
  * 
  * 	    GateDaemon Project
  * 	    Information Technologies/Network Resources
- * 	    200 CCC, Garden Avenue
+ * 	    200 CCC
  * 	    Cornell University
  * 	    Ithaca, NY  14853-2601  USA
  * 
@@ -1078,11 +1304,6 @@ ospf_rxpkt __PF4(ip, struct ip *,
  * 	Please forward bug fixes, enhancements and questions to the
  * 	gated mailing list: gated-people@gated.cornell.edu.
  * 
- * 	Authors:
- * 
- * 		Jeffrey C Honig <jch@gated.cornell.edu>
- * 		Scott W Brim <swb@gated.cornell.edu>
- * 
  * ------------------------------------------------------------------------
  * 
  *       Portions of this software may fall under the following
