@@ -1,10 +1,94 @@
 /*
- * ospf_tqhandle.c,v 1.13 1993/03/22 02:40:35 jch Exp
+ * ------------------------------------------------------------------------
+ * 
+ * GateD, Release 3.5.5 
+ * 
+ * Copyright (c) 1996, 1997 The Regents of the University of Michigan
+ * All Rights Reserved
+ * 
+ * License to use, copy, modify, and distribute this software and its
+ * documentation can be obtained from Merit at the University of Michigan.
+ * 
+ * Merit GateDaemon Project
+ * 4251 Plymouth Road, Suite C
+ * Ann Arbor, MI 48105
+ * 
+ * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION WARRANTIES OF 
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE REGENTS OF THE
+ * UNIVERSITY OF MICHIGAN AND MERIT DO NOT WARRANT THAT THE
+ * FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET LICENSEE'S REQUIREMENTS OR
+ * THAT OPERATION WILL BE UNINTERRUPTED OR ERROR FREE. The Regents of the
+ * University of Michigan and Merit shall not be liable for
+ * any special, indirect, incidental or consequential damages with respect
+ * to any claim by Licensee or any third party arising from use of the
+ * software. GateDaemon was originated and developed through release 3.0
+ * by Cornell University and its collaborators.
+ * 
+ * Please forward bug fixes, enhancements and questions to the
+ * gated mailing list: gated-people@gated.merit.edu.
+ * 
+ * ---------------------
+ * 
+ * Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.
+ * All rights reserved.
+ * 
+ * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY
+ * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
+ * LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE.
+ * 
+ * Royalty-free licenses to redistribute GateD Release
+ * 3 in whole or in part may be obtained by writing to:
+ * 
+ * GateDaemon Project
+ * Information Technologies/Network Resources
+ * 200 CCC
+ * Cornell University
+ * Ithaca, NY  14853-2601  USA
+ * 
+ * GateD is based on Kirton's EGP, UC Berkeley's routing
+ * daemon	 (routed), and DCN's HELLO routing Protocol.
+ * Development of GateD has been supported in part by the
+ * National Science Foundation.
+ * 
+ * Please forward bug fixes, enhancements and questions to the
+ * gated mailing list: gated-people@gated.cornell.edu.
+ * 
+ * ------------------------------------------------------------------------
+ * 
+ * Portions of this software may fall under the following
+ * copyrights:
+ * 
+ * Copyright (c) 1988 Regents of the University of California.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms are
+ * permitted provided that the above copyright notice and
+ * this paragraph are duplicated in all such forms and that
+ * any documentation, advertising materials, and other
+ * materials related to such distribution and use
+ * acknowledge that the software was developed by the
+ * University of California, Berkeley.  The name of the
+ * University may not be used to endorse or promote
+ * products derived from this software without specific
+ * prior written permission.  THIS SOFTWARE IS PROVIDED
+ * ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * This copyright has ben automaticly added by the util/addcopyright.pl program.
+ * __END_OF_COPYRIGHT__
  */
 
-/* Gated Release 3.0 */
-/* Copyright (c) 1990,1991,1992,1993 by Cornell University. All rights reserved. */
-/* Refer to Particulars and other Copyright notices at the end of this file. */
+/*
+ * ospf_tqhandle.c,v 1.22.2.8 1994/09/20 15:48:57 jch Exp
+ */
+
+/* Gated Release 3.5 */
+/* Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.  All */
+/* rights reserved.  Refer to Particulars and other Copyright notices at */
+/* the end of this file.  */
+/*  */
 
 
 #define	INCLUDE_TIME
@@ -47,11 +131,13 @@ void
 ospf_ifdown __PF1(intf, struct INTF *)
 {
     struct AREA *area = intf->area;
-    struct LSDB_LIST *txq = LLNULL;
+    struct ospf_lsdb_list *txq = LLNULL;
 
-    trace(TR_OSPF, 0, "ospf_ifdown: Interface %s (%s) DOWN",
-	  intf->ifap->ifa_name,
-	  intf->ifap->ifa_name);
+    trace_only_tf(ospf.trace_options,
+		  0,
+		  ("ospf_ifdown: Interface %s (%s) DOWN",
+		   intf->ifap->ifa_link->ifl_name,
+		   intf->ifap->ifa_link->ifl_name));
 
     (*if_trans[INTF_DOWN][intf->state]) (intf);
     virtual_intf_down(intf);
@@ -72,10 +158,10 @@ ospf_ifdown __PF1(intf, struct INTF *)
     }
     if (txq != LLNULL) {	/* may be locked out */
 	self_orig_area_flood(area, txq, LS_RTR);
-	ospf_freeq((struct Q **)&txq, OMEM_LL);
+	ospf_freeq((struct Q **)&txq, ospf_lsdblist_index);
     }
     if (area->spfsched)
-	run_spf(area, 0);
+	ospf_spf_sched();
 }
 
 
@@ -85,7 +171,7 @@ ospf_ifdown __PF1(intf, struct INTF *)
 void
 ospf_ifup __PF1(intf, struct INTF *)
 {
-    struct LSDB_LIST *txq = LLNULL;
+    struct ospf_lsdb_list *txq = LLNULL;
     struct AREA *area = intf->area;
 
     (*(if_trans[INTF_UP][IDOWN])) (intf);
@@ -102,74 +188,115 @@ ospf_ifup __PF1(intf, struct INTF *)
     }
     if (txq != LLNULL) {	/* may be locked out */
 	self_orig_area_flood(area, txq, LS_RTR);
-	ospf_freeq((struct Q **)&txq, OMEM_LL);
+	ospf_freeq((struct Q **)&txq, ospf_lsdblist_index);
     }
     if (area->spfsched)
-	run_spf(area, 0);
+	ospf_spf_sched();
 }
 
 
 /*ARGSUSED*/
 void
-tq_hellotmr __PF2(t, OTIMER *,
+tq_hellotmr __PF2(tip, task_timer *,
 		  interval, time_t)
 {
-    struct INTF *intf = INTF_FROM_TIMER(t);
-    struct NBR *nbr;
-    struct LSDB_LIST *txq = LLNULL;
-    struct AREA *area = intf->area;
-    int do_poll = 0;
+    struct INTF *intf = (struct INTF *) tip->task_timer_data;
+
+    TRAP_REF_UPDATE;	/* Update the trap event counter */
+
+    if (intf->state <= IDOWN) {
+	return;
+    }
+
+    send_hello(intf, 0, FALSE);
+
+    if (intf->type == NONBROADCAST) {
+	if ((time_t) (intf->pollmod * intf->hello_timer) >= intf->poll_timer) {
+	    struct NBR *nbr;
+		
+	    NBRS_LIST(nbr, intf) {	    
+		/* Poll timer */
+		if (nbr->state == NDOWN) {
+		    send_hello(intf, nbr, FALSE);
+		}
+	    } NBRS_LIST_END(nbr, intf) ;
+
+	    intf->pollmod = 1;
+	} else {
+	    intf->pollmod++;
+	}
+    }
+}
+
+
+/*ARGSUSED*/
+void
+tq_adjtmr __PF2(tip, task_timer *,
+		interval, time_t)
+{
+    struct INTF *intf = (struct INTF *) tip->task_timer_data;
+    time_t next_time = intf->dead_timer;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
     if (intf->state > IDOWN) {
+	struct NBR *nbr;
+	struct ospf_lsdb_list *txq = LLNULL;
+	struct AREA *area = intf->area;
 	struct NBR *next_nbr;
- 	if (intf->type == NONBROADCAST) {
- 	    if ((intf->pollmod * intf->hello_timer) >= intf->poll_timer) {
- 		do_poll = TRUE;
- 		intf->pollmod = 1;
- 	    } else {
- 	    	intf->pollmod++;
- 	    }
- 	}
- 	
-	send_hello(intf, 0, FALSE);
+
 	for (nbr = FirstNbr(intf); nbr != NBRNULL; nbr = next_nbr) {
 	    next_nbr = nbr->next;
+	    
 	    /* Inactivity timer */
-	    if ((nbr->state >= NATTEMPT) &&
-		nbr->last_hello &&
-		(time_sec - nbr->last_hello >= intf->dead_timer)) {
-		nbr->last_hello = 0;
-		if (intf->type == VIRTUAL_LINK) {
-		    (*if_trans[INTF_DOWN][intf->state]) (intf);
-		    goto virtual_bail_out;
-		} else
-		    (*(nbr_trans[INACT_TIMER][nbr->state])) (intf, nbr);
+	    if (nbr->state >= NATTEMPT
+		&& nbr->last_hello) {
+		time_t left = intf->dead_timer - (time_sec - nbr->last_hello);
+
+		if (left <= 0) {
+		    nbr->last_hello = 0;
+		    if (intf->type == VIRTUAL_LINK) {
+			(*if_trans[INTF_DOWN][intf->state]) (intf);
+			goto virtual_bail_out;
+		    } else {
+			(*(nbr_trans[INACT_TIMER][nbr->state])) (intf, nbr);
+		    }
+		} else if (left < next_time) {
+		    next_time = left;
+		}
 	    }
+	    
 	    /*
 	     * Handle holding interval for Slave mode - nbr state is Loading
 	     * or Full
 	     */
-	    if (nbr->mode == SLAVE_HOLD &&
-		nbr->last_exch &&
-		(time_sec - nbr->last_exch) > intf->dead_timer) {
-		nbr->mode = SLAVE;
-		if (!(nbr->dbsum == LSDB_SUM_NULL)) {
-		    DB_FREE_PKT(nbr->dbsum);
-		    FREE(nbr->dbsum, OMEM_DBSUM);
-		    nbr->dbsum = LSDB_SUM_NULL;
+	    if (nbr->mode == SLAVE_HOLD
+		&& nbr->last_exch) {
+		time_t left = intf->dead_timer - (time_sec - nbr->last_exch);
+
+		if (left <= 0) {
+		    nbr->mode = SLAVE;
+		    if (!(nbr->dbsum == LSDB_SUM_NULL)) {
+			dbsum_free(nbr->dbsum);
+			nbr->dbsum = LSDB_SUM_NULL;
+		    }
+		} else if (left < next_time) {
+		    next_time = left;
 		}
 	    }
-	    /* Poll timer */
- 	    if (do_poll && nbr->state == NDOWN)
- 		send_hello(intf, nbr, FALSE);
  	}
+
 	/* Wait timer */
-	if (intf->state == IWAITING &&
-	    intf->wait_time &&
-	    (time_sec - intf->wait_time) >= intf->dead_timer)
-	    (*if_trans[WAIT_TIMER][intf->state]) (intf);
+	if (intf->state == IWAITING
+	    && intf->wait_time) {
+	    time_t left = intf->wait_time - (time_sec - intf->dead_timer);
+
+	    if (left <= 0) {
+		(*if_trans[WAIT_TIMER][intf->state]) (intf);
+	    } else if (left < next_time) {
+		next_time = left;
+	    }
+	}
 
 	if (BIT_TEST(intf->flags, OSPF_INTFF_NBR_CHANGE)) {
 	    (*(if_trans[NBR_CHANGE][intf->state])) (intf);
@@ -180,6 +307,7 @@ tq_hellotmr __PF2(t, OTIMER *,
 	    area->spfsched |= build_net_lsa(intf, &txq, 0);
 	    BIT_RESET(intf->flags, OSPF_INTFF_BUILDNET);
 	}
+
       virtual_bail_out:
 	if (area->build_rtr) {
 	    area->spfsched |= build_rtr_lsa(area, &txq, 0);
@@ -187,11 +315,14 @@ tq_hellotmr __PF2(t, OTIMER *,
 	}
 	if (txq != LLNULL) {	/* may be locked out */
 	    self_orig_area_flood(area, txq, LS_RTR);
-	    ospf_freeq((struct Q **)&txq, OMEM_LL);
+	    ospf_freeq((struct Q **)&txq, ospf_lsdblist_index);
 	}
 	if (area->spfsched)
-	    run_spf(area, 0);
+	    ospf_spf_sched();
     }
+
+    /* Reschedule */
+    task_timer_set(tip, (time_t) 0, next_time);
 }
 
 
@@ -201,23 +332,23 @@ tq_hellotmr __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_lsa_lock __PF2(t, OTIMER *,
+tq_lsa_lock __PF2(tip, task_timer *,
 		  interval, time_t)
 {
-    struct AREA *a = AREA_FROM_TIMER(t);
+    struct AREA *a = (struct AREA *) tip->task_timer_data;
     struct INTF *intf;
-    struct LSDB_LIST *txq = LLNULL;
-    struct LSDB *db, *dbnext;
+    struct ospf_lsdb_list *txq = LLNULL;
+    struct LSDB *db;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
     /*
      * Check to-be-free list
      */
-    for (db = DB_PTR(&(ospf.db_free_list))[NEXT]; db; db = dbnext) {
-	dbnext = DB_PTR(db)[NEXT];
-	if (DB_CAN_BE_FREED(db))
-	    db_free(db);
-    }
+    DB_RUNQUE(ospf.db_free_list, db) {
+	if (DB_CAN_BE_FREED(db)) {
+	    db_free(db, LS_TYPE(db));
+	}
+    } DB_RUNQUE_END(ospf.db_free_list, db) ;
 
     /* MODIFIED 1/22/92 */
     /* If LSDB limit is exceeded send trap */
@@ -259,14 +390,14 @@ tq_lsa_lock __PF2(t, OTIMER *,
     }
     if (txq != LLNULL) {	/* may not be adjacent to anyone */
 	self_orig_area_flood(a, txq, LS_NET);
-	ospf_freeq((struct Q **)txq, OMEM_LL);
+	ospf_freeq((struct Q **)txq, ospf_lsdblist_index);
     }
     /*
      * Check for a spf sched event having been run - if out of memory during
      * spf, spf may have scheduled
      */
     if (a->spfsched)
-	run_spf(a, 0);
+	ospf_spf_sched();
 }
 
 
@@ -276,12 +407,13 @@ tq_lsa_lock __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_IntLsa __PF2(t, OTIMER *,
+tq_IntLsa __PF2(tip, task_timer *,
 		interval, time_t)
 {
+    int schedule = 0;
     struct AREA *a;
     struct INTF *intf;
-    struct LSDB_LIST *txq = LLNULL;
+    struct ospf_lsdb_list *txq = LLNULL;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
@@ -295,12 +427,17 @@ tq_IntLsa __PF2(t, OTIMER *,
 
 	if (txq != LLNULL) {	/* may be locked out */
 	    self_orig_area_flood(a, txq, LS_RTR);
-	    ospf_freeq((struct Q **)&txq, OMEM_LL);
+	    ospf_freeq((struct Q **)&txq, ospf_lsdblist_index);
 	}
 	if (a->spfsched)
-	    run_spf(a, 0);
+	    schedule++;
     } AREA_LIST_END(a) ;
-    timer_interval(t, LSRefreshTime);
+
+    if (schedule) {
+	ospf_spf_sched();
+    }
+    
+    task_timer_set_interval(tip, LSRefreshTime);
 }
 
 
@@ -310,7 +447,7 @@ tq_IntLsa __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_SumLsa __PF2(t, OTIMER *,
+tq_SumLsa __PF2(tip, task_timer *,
 		interval, time_t)
 {
     int reset = LSRefreshTime;
@@ -324,143 +461,20 @@ tq_SumLsa __PF2(t, OTIMER *,
 	reset = 40;
     /* If ran out of buffers - try again in 40 seconds */
 
-    timer_interval(t, reset);
+    task_timer_set_interval(tip, reset);
 }
 
 
-/*
- *	LSA update interval timer for ase LSAs
- *	- Walk linked list of self-originated LSAs and generate new versions
- */
-/*ARGSUSED*/
-void
-tq_AseLsa __PF2(t, OTIMER *,
-		interval, time_t)
-{
-    int i;
-    struct LSDB *db, *last = LSDBNULL;
-    time_t reset = ospf.export_interval;  /* reset value for this timer */
-    u_int16 age;
-#ifdef NOTYETDEF
-    struct LSDB *start_flush, *end_flush;  /* For flushing when lsdb overflow */
-#endif
-
-    TRAP_REF_UPDATE;	/* Update the trap event counter */
-
-    /* If no externals just return */
-    if (!DB_PTR(&ospf.my_ase_list)[NEXT]) {
-    	    timer_interval(t, reset);
-	    return;
-    }
-
-    /* note start time of this period to know when to resched */
-    if (ospf.cur_ase == LSDBNULL) {
-	ospf.ase_start = time_sec;
-	ospf.cur_ase = DB_PTR(&ospf.my_ase_list)[NEXT];
-    }
-
-#ifdef NOTYETDEF
-    start_flush = ospf.cur_ase;		/* Used if db overflow */
-#endif
-
-    /* 
-     * set new seq and newly generated time 
-     */
-    for (i = 0, db = ospf.cur_ase;
-	 db != LSDBNULL && i < ASEGenLimit;
-	 db = DB_PTR(db)[NEXT], i++) {
-	if (NO_GUTS(db))
-	    continue;
-	if ((time_sec - ADV_AGE(db)) < MinLSInterval)
-	    continue;
-
-	if (DB_RETRANS(db) != NLNULL)
-	    rem_db_retrans(db);
-
-	LS_AGE(db) = 0;
-#ifdef NOTYETDEF
-	/* MODIFIED 1/22/92 */
-	if (ospf.db_overflow) {
-	    age = MaxAge;
-	} else
-#endif
-	if ((LS_SEQ(db) == MaxSeqNum) &&
-	    (DB_SEQ_MAX(db) == FALSE) &&
-	    (ospf.nbrEcnt)) {
-	    /* Not noted - flush from everyone's LSDB */
-	    DB_SEQ_MAX(db) = TRUE;
-	    age = MaxAge;
-	} else {
-	    DB_SEQ_MAX(db) = FALSE;
-	    LS_SEQ(db) = NEXTNSEQ(LS_SEQ(db));
-	    age = 0;
-	}
-    	ospf.db_chksumsum -= LS_CKS(db);
-	ospf_checksum_sum(DB_ASE(db), ASE_LA_HDR_SIZE, ospf.db_chksumsum);
-    	ospf.orig_new_lsa++;
-	DB_TIME(db) = time_sec;
-	LS_AGE(db) = age;
-
-	if (BIT_TEST(ospf.trace_flags, TR_OSPF_LSA_BLD)) {
-	    ospf_trace_build((struct AREA *) 0, (struct AREA *) 0, DB_ADV(db), FALSE);
-	}
-	last = db;
-    }
-
-#ifdef NOTYETDEF
-    end_flush = last;
-#endif
-    /* 
-     * send LS_ASEs from current to last
-     */
-    if (ospf.cur_ase != LSDBNULL && last)
-	send_ase(ospf.cur_ase, last);
- 
-    /* 
-     * If more to send, set reset time to be MinAseInterval, else
-     * set it to be LSRefreshTime - elapsed time 
-     */
-    if ((!last) || (last && (DB_PTR(last)[NEXT] == LSDBNULL))) {
-	last = LSDBNULL;
-	reset = (LSRefreshTime - (time_sec - ospf.ase_start));
-	if (reset <= 0)
-    	    reset = ospf.export_interval;  /* reset value for this timer */
-    } else {
-    	last = DB_PTR(last)[NEXT];
-    }
-
-    ospf.cur_ase = last;
-
-#ifdef NOTYETDEF
-    for (db = start_flush; 
-	 db != LSDBNULL;
-	 db = DB_PTR(db)[NEXT]) 
-    {
-	if (NO_GUTS(db))
-	    continue;
-	if ((time_sec - ADV_AGE(db)) < MinLSInterval)
-	    continue;
-	DEL_DBQ(db);
-	DB_WHERE(db) = ON_ASE_INFINITY;
-	DB_FREEME(db) = TRUE;
-	ADD_DBQ(&ospf.db_free_list, db);
-	if (db == end_flush) break;
-    }
-#endif
-
-    timer_interval(t, reset);
-}
-
 /*
  *	retransmit timer for this interface
  *		what to retransmit is based on state
  */
 /*ARGSUSED*/
 void
-tq_retrans __PF2(t, OTIMER *,
+tq_retrans __PF2(tip, task_timer *,
 		 interval, time_t)
 {
-    struct INTF *intf = INTF_FROM_TIMER(t);
+    struct INTF *intf = (struct INTF *) tip->task_timer_data;
     struct NBR *n;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
@@ -500,18 +514,17 @@ tq_retrans __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_ack __PF2(t, OTIMER *,
+tq_ack __PF2(tip, task_timer *,
 	     interval, time_t)
 {
     int acks = 0;
     struct AREA *area;
+    struct INTF *intf;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
     /* Send acks for all interfaces */
     AREA_LIST(area) {
-	struct INTF *intf;
-
 	INTF_LIST(intf, area) {
 	    if (intf->acks.ptr[NEXT]) {
 		acks += send_ack(intf, NBRNULL, &intf->acks);	/* non-direct ack */
@@ -521,8 +534,6 @@ tq_ack __PF2(t, OTIMER *,
 
     /* Send acks for all virtual interfaces */
     if (IAmBorderRtr && ospf.vcnt) {
-	struct INTF *intf;
-
 	VINTF_LIST(intf) {
 	    if (intf->acks.ptr[NEXT]) {
 		acks += send_ack(intf, NBRNULL, &intf->acks);	/* non-direct ack */
@@ -533,7 +544,7 @@ tq_ack __PF2(t, OTIMER *,
     if (!acks) {
 	/* No acks left, stop timer */
 
-	timer_reset(t);
+	task_timer_reset(tip);
     }
 }
 
@@ -541,12 +552,12 @@ tq_ack __PF2(t, OTIMER *,
 static int
 dbage  __PF5(a, struct AREA *,
 	     type, int,
-	     trans, struct LSDB_LIST **,
+	     trans, struct ospf_lsdb_list **,
 	     start, int,
 	     stop, int)
 {
-    struct LSDB *hp, *db, *dblast;
-    struct LSDB_LIST *ll;
+    register struct LSDB_HEAD *hp;
+    struct ospf_lsdb_list *ll;
     u_int16 age;
     int spfsched = 0;
 
@@ -556,88 +567,75 @@ dbage  __PF5(a, struct AREA *,
      * Age DB from start to stop
      * - Ase can age pieces at a time to avoid major floods
      */
-    for (hp = &(a->htbl[type][start]);
-	 hp < &(a->htbl[type][stop]);
-	 hp++) {
-	for (dblast = hp, db = DB_NEXT(hp);
-	     db;
-	     dblast = db, db = DB_NEXT(db)) {
-	    if (NO_GUTS(db)) {
-		if (type == LS_ASE) {
-		    /* garbage collection, remove all traces of */
-		    DB_NEXT(dblast) = DB_NEXT(db);
-		    FREE(db, OMEM_LSDB);
-		    db = dblast;
-		}
-		continue;	/* was freed */
-	    }
+    LSDB_HEAD_LIST(a->htbl[type], hp, start, stop) {
+	register struct LSDB *db;
+
+	LSDB_LIST(hp, db) {
 	    /* MaxSeq and no acks, generate a new one */
-	    if (DB_SEQ_MAX(db) == TRUE &&
-		DB_RETRANS(db) == NLNULL &&
-		DB_FREEME(db) != TRUE) {
-		spfsched |= beyond_max_seq(a, INTFNULL, db, trans, trans, TRUE);
-	    } else if (DB_WHERE(db) != ON_ASE_LIST &&
-		     ADV_AGE(db) >= MaxAge &&
-		     !(DB_SEQ_MAX(db))) {
+	    if (db->lsdb_seq_max
+		&& !db->lsdb_retrans
+		&& !DB_FREEME(db)) {
+		/* Need to re-originate */
+		
+		spfsched |= beyond_max_seq(a, (struct INTF *) 0, db, trans, trans, TRUE);
+	    } else if (DB_WHERE(db) != ON_ASE_LIST
+		       && ADV_AGE(db) >= MaxAge
+		       && !db->lsdb_seq_max) {
 		/* Currently active route? */
-		if (DB_ROUTE(db) || DB_ASB_RTR(db)) {
-		    if ((LS_TYPE(db) == LS_SUM_NET) ||
-			(LS_TYPE(db) == LS_SUM_ASB)) {
-			if ((IAmBorderRtr && a->area_id == OSPF_BACKBONE) ||
-			    (!(IAmBorderRtr)))
-			    spfsched |= SCHED_BIT(type);
-		    } else
+		
+		if (db->lsdb_route || db->lsdb_asb_rtr) {
+		    if ((LS_TYPE(db) != LS_SUM_NET
+			 && LS_TYPE(db) != LS_SUM_ASB)
+			|| !IAmBorderRtr
+			|| a->area_id == OSPF_BACKBONE) {
 			spfsched |= SCHED_BIT(type);
+		    }
 
 		    LS_AGE(db) = MaxAge;
 		    if (DB_FREEME(db) != TRUE) {
-			DEL_DBQ(db);
+			DB_REMQUE(db);
 			DB_FREEME(db) = TRUE;
-			ADD_DBQ(&ospf.db_free_list, db);
+			DB_ADDQUE(ospf.db_free_list, db);
 		    }
-		    LL_ALLOC(ll);
+		    ll = (struct ospf_lsdb_list *) task_block_alloc(ospf_lsdblist_index);
 		    ll->lsdb = db;
 		    EN_Q((*trans), ll);
-		} else if (DB_FREEME(db)) {	/* was set to be freed */
-		    /* free if conditions are met */
+		} else if (DB_FREEME(db)) {
+		    /* was set to be freed, free if conditions are met */
+
 		    if (DB_CAN_BE_FREED(db)) {
-			db_free(db);
-			/*
-			 * Asbr rtr entries are pointed to by externals so
-			 * they shouldn't be freed
-			 */
-			if ((NO_GUTS(db)) && (type == LS_ASE)) {
-			    /* garbage collection, remove all traces of */
-			    DB_NEXT(dblast) = DB_NEXT(db);
-			    FREE(db, OMEM_LSDB);
-			    db = dblast;
-			}
+			db_free(db, LS_TYPE(db));
 		    }
-		} else {	/* db->freeme hasn't been set yet */
+		} else {
+		    /* db->freeme hasn't been set yet */
+
 		    LS_AGE(db) = MaxAge;
-		    DEL_DBQ(db);
+		    DB_REMQUE(db);
 		    DB_FREEME(db) = TRUE;
-		    ADD_DBQ(&ospf.db_free_list, db);
-		    LL_ALLOC(ll);
+		    DB_ADDQUE(ospf.db_free_list, db);
+		    ll = (struct ospf_lsdb_list *) task_block_alloc(ospf_lsdblist_index);
 		    ll->lsdb = db;
 		    EN_Q((*trans), ll);
 		}
 		continue;
 	    }
+
 	    /* Rerun LS checksum */
 	    age = LS_AGE(db);
 	    LS_AGE(db) = 0;
 	    if (ospf_checksum_bad(DB_RTR(db), ntohs(LS_LEN(db)))) {
-		trace(TR_OSPF, 0, "Chksum failed: area %A type %d id %s adv %s",
-		      sockbuild_in(0, a->area_id),
-		      type,
-		      LS_ID(db),
-		      ADV_RTR(db));
+		trace_log_tf(ospf.trace_options, 0, LOG_WARNING,
+		     ("dbage: checksum failed: area %A type %d id %A adv %A",
+			      sockbuild_in(0, a->area_id),
+			      type,
+			      sockbuild_in(0, LS_ID(db)),
+			      sockbuild_in(0, ADV_RTR(db))));
 		assert(FALSE);
 	    }
 	    LS_AGE(db) = age;
-	}
-    }
+	} LSDB_LIST_END(hp, db) ;
+    } LSDB_HEAD_LIST_END(a->htbl[type], hp, start, stop) ;
+
     return spfsched;
 }
 
@@ -651,12 +649,13 @@ dbage  __PF5(a, struct AREA *,
  */
 /*ARGSUSED*/
 void
-tq_int_age __PF2(t, OTIMER *,
+tq_int_age __PF2(t, task_timer *,
 		 interval, time_t)
 {
+    int schedule = 0;
     struct AREA *a;
     /* list of lsdbs to be sent */
-    struct LSDB_LIST *trans = LLNULL;
+    struct ospf_lsdb_list *trans = LLNULL;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
@@ -672,11 +671,15 @@ tq_int_age __PF2(t, OTIMER *,
 	/* if a db entry was newly discovered to be MaxAge, flood it */
 	if (trans != LLNULL) {
 	    self_orig_area_flood(a, trans, LS_RTR);
-	    ospf_freeq((struct Q **)&trans, OMEM_LL);
+	    ospf_freeq((struct Q **)&trans, ospf_lsdblist_index);
 	}
 	if (a->spfsched)
-	    run_spf(a, 0);
+	    schedule++;
     } AREA_LIST_END(a) ;
+
+    if (schedule) {
+	ospf_spf_sched();
+    }
 }
 
 /*
@@ -687,11 +690,12 @@ tq_int_age __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_sum_age __PF2(t, OTIMER *,
+tq_sum_age __PF2(t, task_timer *,
 		 interval, time_t)
 {
+    int schedule = 0;
     struct AREA *a;
-    struct LSDB_LIST *trans = LLNULL;
+    struct ospf_lsdb_list *trans = LLNULL;
 
     TRAP_REF_UPDATE;	/* Update the trap event counter */
 
@@ -703,11 +707,15 @@ tq_sum_age __PF2(t, OTIMER *,
 	 */
 	if (trans != LLNULL) {
 	    self_orig_area_flood(a, trans, LS_SUM_NET);
-	    ospf_freeq((struct Q **)&trans, OMEM_LL);
+	    ospf_freeq((struct Q **)&trans, ospf_lsdblist_index);
 	}
 	if (a->spfsched)
-	    run_spf(a, 0);
+	    schedule++;
     } AREA_LIST_END(a) ;
+
+    if (schedule) {
+	ospf_spf_sched();
+    }
 }
 
 
@@ -717,11 +725,11 @@ tq_sum_age __PF2(t, OTIMER *,
  */
 /*ARGSUSED*/
 void
-tq_ase_age __PF2(t, OTIMER *,
+tq_ase_age __PF2(t, task_timer *,
 		 interval, time_t)
 {
-    struct AREA *area = ospf.area;
-    struct LSDB_LIST *trans = LLNULL;
+    struct AREA *area = ospf.area.area_forw;
+    struct ospf_lsdb_list *trans = LLNULL;
     int ase_age_end = ospf.ase_age_ndx + ASE_AGE_NDX_ADD;
     int ase_age_start = ospf.ase_age_ndx;
 
@@ -752,22 +760,22 @@ tq_ase_age __PF2(t, OTIMER *,
 	    }
 	} AREA_LIST_END(a) ;
 
-	ospf_freeq((struct Q **) &trans, OMEM_LL);
+	ospf_freeq((struct Q **) &trans, ospf_lsdblist_index);
     }
     /*
      * something with a valid route has aged out
      */
     if (area->spfsched)
-	run_spf(area, 0);
+	ospf_spf_sched();
 }
 
 
 /*
  * ------------------------------------------------------------------------
  * 
- * 	GateD, Release 3
+ * 	GateD, Release 3.5
  * 
- * 	Copyright (c) 1990,1991,1992,1993 by Cornell University
+ * 	Copyright (c) 1990,1991,1992,1993,1994,1995 by Cornell University.
  * 	    All rights reserved.
  * 
  * 	THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY
@@ -780,7 +788,7 @@ tq_ase_age __PF2(t, OTIMER *,
  * 
  * 	    GateDaemon Project
  * 	    Information Technologies/Network Resources
- * 	    200 CCC, Garden Avenue
+ * 	    200 CCC
  * 	    Cornell University
  * 	    Ithaca, NY  14853-2601  USA
  * 
@@ -792,11 +800,6 @@ tq_ase_age __PF2(t, OTIMER *,
  * 	Please forward bug fixes, enhancements and questions to the
  * 	gated mailing list: gated-people@gated.cornell.edu.
  * 
- * 	Authors:
- * 
- * 		Jeffrey C Honig <jch@gated.cornell.edu>
- * 		Scott W Brim <swb@gated.cornell.edu>
- * 
  * ------------------------------------------------------------------------
  * 
  *       Portions of this software may fall under the following
